@@ -2,7 +2,7 @@
 *                Exim Monitor                    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2012 */
+/* Copyright (c) University of Cambridge 1995 - 2015 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -145,10 +145,8 @@ BOOL    dkim_disable_verify      = FALSE;
 
 BOOL    dont_deliver           = FALSE;
 
-#ifdef EXPERIMENTAL_DSN
 int     dsn_ret                = 0;
 uschar *dsn_envid              = NULL;
-#endif
 
 #ifdef WITH_CONTENT_SCAN
 int     fake_response          = OK;
@@ -220,6 +218,10 @@ tls_support tls_in = {
  -1,	/* tls_active */
  0,	/* bits */
  FALSE,	/* tls_certificate_verified */
+#ifdef EXPERIMENTAL_DANE
+ FALSE, /* dane_verified */
+ 0,     /* tlsa_usage */
+#endif
  NULL,	/* tls_cipher */
  FALSE,	/* tls_on_connect */
  NULL,	/* tls_on_connect_ports */
