@@ -1,10 +1,8 @@
-/* $Cambridge: exim/src/src/spam.h,v 1.3 2005/04/27 10:00:18 ph10 Exp $ */
-
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) Tom Kistner <tom@duncanthrax.net> 2003-???? */
+/* Copyright (c) Tom Kistner <tom@duncanthrax.net> 2003 - 2015 */
 /* License: GPL */
 
 /* spam defines */
@@ -19,12 +17,21 @@
 
 /* SHUT_WR seems to be undefined on Unixware ? */
 #ifndef SHUT_WR
-#define SHUT_WR 1
+# define SHUT_WR 1
 #endif
 
-typedef struct spamd_address_container {
-  uschar tcp_addr[24];
-  unsigned int tcp_port;
+/* default weight */
+#define SPAMD_WEIGHT 1
+
+typedef struct spamd_address_container
+{
+  uschar * hostspec;
+  int is_rspamd:1;
+  int is_failed:1;
+  unsigned int weight;
+  unsigned int timeout;
+  unsigned int retry;
+  unsigned int priority;
 } spamd_address_container;
 
 #endif
